% Find all connected components of a bipartite graph.

function [nconncomps, conncomps] = find_bipartite_conn_comps(n1, n2, G)

% Recursively label nodes in the graph until all nodes are labeled.
labeled1=zeros(1,n1); labeled2=zeros(1,n2); flag=1;

% Label all the singletons.
k=0;
for i=1:n1
tmp=find(G(i,:)>0);
if (length(tmp)==0)
k=k+1; labeled1(i)=k;
end
end
for i=1:n2
tmp=find(G(:,i)>0);
if (length(tmp)==0)
k=k+1; labeled2(i)=k;
end
end


while (flag==1)

% Check if all nodes are labeled.
n=0;
for i=1:n1
if (labeled1(i)==0)
n=n+1;
end
end
for i=1:n2
if (labeled2(i)==0)
n=n+1;
end
end

% If yes, then return.
if (n==0)
flag=0;

% Otherwise pick up an unlabeled node and recursively label its neighbors.
else
i=1; j=0;
while ((i<=n1)&(j==0))
if (labeled1(i)==0)
j=i;
end
i=i+1;
end
k=k+1;
[labeled1,labeled2]=recurse_label(1,j,n1,n2,G,labeled1,labeled2,k);
end

end


% Find the connected components from the labels.
nconncomps=k; clear conncomps;
for i=1:nconncomps
conncomps{i}.n1=0;
conncomps{i}.comps1=[];
conncomps{i}.n2=0;
conncomps{i}.comps2=[];
end

for i=1:n1
j=labeled1(i);
conncomps{j}.n1=conncomps{j}.n1+1;
conncomps{j}.comps1(conncomps{j}.n1)=i;
end

for i=1:n2
j=labeled2(i);
conncomps{j}.n2=conncomps{j}.n2+1;
conncomps{j}.comps2(conncomps{j}.n2)=i;
end

% Sort the connected components by size.
tmp=zeros(1,nconncomps);
for i=1:nconncomps
tmp(i)=conncomps{i}.n1+conncomps{i}.n2;
end
[Y,I]=sort(tmp,'descend'); clear tmp2;
for i=1:nconncomps
tmp2{i}=conncomps{I(i)}; 
end
conncomps=tmp2;

% Discard the connected components which do not have type 1 or type 2 nodes.
discard=zeros(1,nconncomps);
for i=1:nconncomps
if ((conncomps{i}.n1<=0)|(conncomps{i}.n2<=0))
discard(i)=1;
end
end
tmp=find(discard==0);
for i=1:length(tmp)
tmp2{i}=conncomps{tmp(i)};
end
nconncomps=length(tmp); conncomps=tmp2;

clear tmp tmp2 Y I discard;

