% Generate association modules from the models of individual genes.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load molecular aberration data.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% mRNA expression.
filename='jmrna_cleaned.csv';
[ngenes,nsamples,genes,samples,jmrnadata]=load_highRes_data2(filename,1);

% Segment CNV.
filename='jsegcnv_cleaned_rescaled.csv';
[nsegments,nsamples,segments,samples,jsegcnvdata]=load_highRes_data2(filename,1);

% Mutation.
filename='jmut_cleaned_valid.csv';
[nmutgenes,nsamples,mutgenes,samples,jmuttable]=load_highRes_data2(filename,1);

% Methylation of gene clusters.
filename='jmethl_cleaned_cludata.csv';
[nmethlclus,nsamples,methlclus,samples,jmethlcludata]=load_highRes_data2(filename,1);

% Methylation of individual genes.
filename='jmethl_cleaned_valid.csv';
[nmethlvalidgenes,nsamples,methlvalidgenes,samples,jmethlvaliddata]=load_highRes_data2(filename,1);

% Methylation clusters.
[nmethlclus,methlclus]=load_clusters('jmethl_cleaned_clusters2.txt','cluster');

% Individual mir expression.
filename='jmir_cleaned.csv';
[nmirs,nsamples,mirs,samples,jmirdata]=load_highRes_data2(filename,1);

% Cluster mir expression.
filename='jmir_cleaned_cludata.csv';
[nmirclus,nsamples,mirclus,samples,jmircludata]=load_highRes_data2(filename,1);

% Mir clusters.
[nmirclus,mirclus]=load_clusters('jmir_cleaned_clusters2.txt','cluster');

% TF expression.
filename='jexpanded_TF_mrna_cleaned.csv';
[nTFs,nsamples,TFs,samples,jTFdata]=load_highRes_data2(filename,1);


% Convert gene indices between various data types.

% mRNA and segments.
genechrs=zeros(1,ngenes);
for n=1:ngenes
genechrs(n)=genes{n}.chr;
end
segchrs=zeros(1,nsegments);
for n=1:nsegments
segchrs(n)=segments{n}.chr;
end

% mRNA and mutated genes.
mut2gene=zeros(1,nmutgenes); gene2mut=zeros(1,ngenes); curind=1;
for n=1:nmutgenes
k=curind; flag=1;
while ((k<=ngenes)&(flag==1))
if (strcmp(genes{k}.genename,mutgenes{n}.genename)==1)
mut2gene(n)=k; gene2mut(k)=n; flag=0; curind=k+1;
elseif (genes{k}.chr>mutgenes{n}.chr)
flag=0; curind=k;
end
k=k+1;
end
end

% mRNA and methylated genes.
methl2gene=zeros(1,nmethlvalidgenes); gene2methl=zeros(1,ngenes); curind=1;
for n=1:nmethlvalidgenes
k=curind; flag=1;
while ((k<=ngenes)&(flag==1))
if (strcmp(genes{k}.genename,methlvalidgenes{n}.genename)==1)
methl2gene(n)=k; gene2methl(k)=n; flag=0; curind=k+1;
elseif (genes{k}.chr>methlvalidgenes{n}.chr)
flag=0; curind=k;
end
k=k+1;
end
end

% mRNA and TF genes.
TF2gene=zeros(1,nTFs); gene2TF=zeros(1,ngenes); curind=1;
for n=1:nTFs
k=curind; flag=1;
while ((k<=ngenes)&(flag==1))
if (strcmp(genes{k}.genename,TFs{n}.genename)==1)
TF2gene(n)=k; gene2TF(k)=n; flag=0; curind=k+1;
elseif (genes{k}.chr>TFs{n}.chr)
flag=0; curind=k;
end
k=k+1;
end
end

% methylation and TF genes.
methl2TF=zeros(1,nmethlvalidgenes); TF2methl=zeros(1,nTFs);
for n=1:nmethlvalidgenes
k=methl2gene(n); l=gene2TF(k);
if (l>0)
methl2TF(n)=l; TF2methl(l)=n;
end
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load integrated association models for individual mRNA genes.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Consider levels 1-4 models.

clear jassociations;
for n=1:ngenes
jassociations{n}.ndrivers=0;
jassociations{n}.drivers={};
end

filename='level4_jmrnaintegrated.csv';
fp=fopen(filename,'r'); flag=1;
while (flag==1)
s=fgets(fp);
if ((length(s)<=0)|(s==-1))
flag=0;
else
drivertype=getitemval3(s,0,9);
item=getitemval3(s,1,9); driverind=atof2(item);
item=getitemval3(s,2,9); regind=atof2(item);
item=getitemval3(s,3,9); passengerind=atof2(item);
item=getitemval3(s,4,9); direction=atof2(item);
item=getitemval3(s,5,9); ll=atof2(item);
item=getitemval3(s,6,9); pval=atof2(item);
item=getitemval3(s,7,9); rep=atof2(item);
jassociations{passengerind}.ndrivers=jassociations{passengerind}.ndrivers+1;
jassociations{passengerind}.drivers{jassociations{passengerind}.ndrivers}.type=drivertype;
jassociations{passengerind}.drivers{jassociations{passengerind}.ndrivers}.ind=driverind;
jassociations{passengerind}.drivers{jassociations{passengerind}.ndrivers}.dir=direction;
if (strcmp(drivertype,'inter segment CNV')==1)
jassociations{passengerind}.drivers{jassociations{passengerind}.ndrivers}.regind=regind;
elseif (strcmp(drivertype,'inter snp')==1)
jassociations{passengerind}.drivers{jassociations{passengerind}.ndrivers}.regind=regind;
else
jassociations{passengerind}.drivers{jassociations{passengerind}.ndrivers}.regind=0;
end
jassociations{passengerind}.drivers{jassociations{passengerind}.ndrivers}.ll=ll;
jassociations{passengerind}.drivers{jassociations{passengerind}.ndrivers}.pval=pval;
jassociations{passengerind}.drivers{jassociations{passengerind}.ndrivers}.rep=rep;
end
end
fclose(fp);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Build mRNA association modules of single molecular aberrations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Initialize association modules.
nmodules=0; modules={};

% Build intra-segment CNV association modules for all chromosomes.
for chr=1:24
modules{chr}.drivertype='intra segment CNV';
modules{chr}.chr=chr;
modules{chr}.ndrivers=0;
modules{chr}.driverinds=[];
modules{chr}.drivers={};
modules{chr}.direction=1;
modules{chr}.nregs=0;
modules{chr}.reginds=[];
modules{chr}.regs={};
modules{chr}.npassengers=0;
modules{chr}.passengerinds=[];
modules{chr}.passengers={};
modules{chr}.lls=[];
modules{chr}.pvals=[];
end
nmodules=24;

for n=1:ngenes
flag=0; i=1; k=0;
while ((i<=jassociations{n}.ndrivers)&(flag==0))
if (strcmp(jassociations{n}.drivers{i}.type,'intra segment CNV')==1)
flag=1; k=i;
end
i=i+1;
end
if (flag==1)
ind=jassociations{n}.drivers{k}.ind;
chr=segments{ind}.chr;
if (ismember(ind,modules{chr}.driverinds)==0)
modules{chr}.ndrivers=modules{chr}.ndrivers+1;
modules{chr}.driverinds(modules{chr}.ndrivers)=ind;
modules{chr}.drivers{modules{chr}.ndrivers}='';
end
modules{chr}.npassengers=modules{chr}.npassengers+1;
modules{chr}.passengerinds(modules{chr}.npassengers)=n;
modules{chr}.passengers{modules{chr}.npassengers}=genes{n}.genename;
modules{chr}.lls(modules{chr}.npassengers)=jassociations{n}.drivers{k}.ll;
modules{chr}.pvals(modules{chr}.npassengers)=jassociations{n}.drivers{k}.pval;
end
end


% Build inter-segment CNV association modules for selected chromosomes.

% Positive association modules.
filename='selposchrs.txt';
selposchrs=load(filename);

for n=1:length(selposchrs)
chr=selposchrs(n);
modules{n+nmodules}.drivertype='inter segment CNV';
modules{n+nmodules}.chr=chr;
modules{n+nmodules}.ndrivers=0;
modules{n+nmodules}.driverinds=[];
modules{n+nmodules}.drivers={};
modules{n+nmodules}.direction=1;
modules{n+nmodules}.nregs=0;
modules{n+nmodules}.reginds=[];
modules{n+nmodules}.regs={};
modules{n+nmodules}.npassengers=0;
modules{n+nmodules}.passengerinds=[];
modules{n+nmodules}.passengers={};
modules{n+nmodules}.lls=[];
modules{n+nmodules}.pvals=[];
end

for n=1:ngenes
for i=1:jassociations{n}.ndrivers
if ((strcmp(jassociations{n}.drivers{i}.type,'inter segment CNV')==1)&(jassociations{n}.drivers{i}.dir==1))
k=i;
ind=jassociations{n}.drivers{k}.ind;
regind=jassociations{n}.drivers{k}.regind;
chr=segments{ind}.chr; modind=find(selposchrs==chr); modind=modind+nmodules;
if (ismember(ind,modules{modind}.driverinds)==0)
modules{modind}.ndrivers=modules{modind}.ndrivers+1;
modules{modind}.driverinds(modules{modind}.ndrivers)=ind;
modules{modind}.drivers{modules{modind}.ndrivers}='';
end
if (ismember(regind,modules{modind}.reginds)==0)
modules{modind}.nregs=modules{modind}.nregs+1;
modules{modind}.reginds(modules{modind}.nregs)=regind;
modules{modind}.regs{modules{modind}.nregs}=TFs{regind}.genename;
end
if (ismember(n,modules{modind}.passengerinds)==0)
modules{modind}.npassengers=modules{modind}.npassengers+1;
modules{modind}.passengerinds(modules{modind}.npassengers)=n;
modules{modind}.passengers{modules{modind}.npassengers}=genes{n}.genename;
modules{modind}.lls(modules{modind}.npassengers)=jassociations{n}.drivers{k}.ll;
modules{modind}.pvals(modules{modind}.npassengers)=jassociations{n}.drivers{k}.pval;
end
end
end
end
nmodules=nmodules+length(selposchrs);


% Negative association modules.
filename='selnegchrs.txt';
selnegchrs=load(filename);

for n=1:length(selnegchrs)
chr=selnegchrs(n);
modules{n+nmodules}.drivertype='inter segment CNV';
modules{n+nmodules}.chr=chr;
modules{n+nmodules}.ndrivers=0;
modules{n+nmodules}.driverinds=[];
modules{n+nmodules}.drivers={};
modules{n+nmodules}.direction=-1;
modules{n+nmodules}.nregs=0;
modules{n+nmodules}.reginds=[];
modules{n+nmodules}.regs={};
modules{n+nmodules}.npassengers=0;
modules{n+nmodules}.passengerinds=[];
modules{n+nmodules}.passengers={};
modules{n+nmodules}.lls=[];
modules{n+nmodules}.pvals=[];
end

for n=1:ngenes
for i=1:jassociations{n}.ndrivers
if ((strcmp(jassociations{n}.drivers{i}.type,'inter segment CNV')==1)&(jassociations{n}.drivers{i}.dir==-1))
k=i;
ind=jassociations{n}.drivers{k}.ind;
regind=jassociations{n}.drivers{k}.regind;
chr=segments{ind}.chr; modind=find(selnegchrs==chr); modind=modind+nmodules;
if (ismember(ind,modules{modind}.driverinds)==0)
modules{modind}.ndrivers=modules{modind}.ndrivers+1;
modules{modind}.driverinds(modules{modind}.ndrivers)=ind;
modules{modind}.drivers{modules{modind}.ndrivers}='';
end
if (ismember(regind,modules{modind}.reginds)==0)
modules{modind}.nregs=modules{modind}.nregs+1;
modules{modind}.reginds(modules{modind}.nregs)=regind;
modules{modind}.regs{modules{modind}.nregs}=TFs{regind}.genename;
end
if (ismember(n,modules{modind}.passengerinds)==0)
modules{modind}.npassengers=modules{modind}.npassengers+1;
modules{modind}.passengerinds(modules{modind}.npassengers)=n;
modules{modind}.passengers{modules{modind}.npassengers}=genes{n}.genename;
modules{modind}.lls(modules{modind}.npassengers)=jassociations{n}.drivers{k}.ll;
modules{modind}.pvals(modules{modind}.npassengers)=jassociations{n}.drivers{k}.pval;
end
end
end
end
nmodules=nmodules+length(selnegchrs);


% Build mutation association modules.
clear mutpospassengers mutnegpassengers;
for n=1:nmutgenes
mutpospassengers{n}=[]; mutnegpassengers{n}=[]; 
end

for n=1:ngenes
for i=1:jassociations{n}.ndrivers
if (strcmp(jassociations{n}.drivers{i}.type,'mutation')==1)
k=jassociations{n}.drivers{i}.ind;
if (jassociations{n}.drivers{i}.dir==1)
mutpospassengers{k}(1+length(mutpospassengers{k}))=n;
elseif (jassociations{n}.drivers{i}.dir==-1)
mutnegpassengers{k}(1+length(mutnegpassengers{k}))=n;
end
end
end
end

for mutind=1:nmutgenes
if (length(mutpospassengers{mutind})>=10)
nmodules=nmodules+1;
modules{nmodules}.drivertype='mutation';
modules{nmodules}.chr=0;
modules{nmodules}.driverinds(1)=mutind;
modules{nmodules}.drivers{1}=mutgenes{mutind}.genename;
modules{nmodules}.direction=1;
modules{nmodules}.nregs=0;
modules{nmodules}.reginds=[];
modules{nmodules}.regs={};
modules{nmodules}.npassengers=length(mutpospassengers{mutind});
modules{nmodules}.passengerinds=mutpospassengers{mutind};
for i=1:modules{nmodules}.npassengers
j=modules{nmodules}.passengerinds(i);
modules{nmodules}.passengers{i}=genes{j}.genename;
k=1; l=0;
while ((k<=jassociations{j}.ndrivers)&(l==0))
if ((strcmp(jassociations{j}.drivers{k}.type,'mutation')==1)&(jassociations{j}.drivers{k}.ind==mutind))
l=k;
end
k=k+1;
end
modules{nmodules}.lls(i)=jassociations{j}.drivers{l}.ll;
modules{nmodules}.pvals(i)=jassociations{j}.drivers{l}.pval;
end
end

if (length(mutnegpassengers{mutind})>=10)
nmodules=nmodules+1;
modules{nmodules}.drivertype='mutation';
modules{nmodules}.chr=0;
modules{nmodules}.driverinds(1)=mutind;
modules{nmodules}.drivers{1}=mutgenes{mutind}.genename;
modules{nmodules}.direction=-1;
modules{nmodules}.nregs=0;
modules{nmodules}.reginds=[];
modules{nmodules}.regs={};
modules{nmodules}.npassengers=length(mutnegpassengers{mutind});
modules{nmodules}.passengerinds=mutnegpassengers{mutind};
for i=1:modules{nmodules}.npassengers
j=modules{nmodules}.passengerinds(i);
modules{nmodules}.passengers{i}=genes{j}.genename;
k=1; l=0;
while ((k<=jassociations{j}.ndrivers)&(l==0))
if ((strcmp(jassociations{j}.drivers{k}.type,'mutation')==1)&(jassociations{j}.drivers{k}.ind==mutind))
l=k;
end
k=k+1;
end
modules{nmodules}.lls(i)=jassociations{j}.drivers{l}.ll;
modules{nmodules}.pvals(i)=jassociations{j}.drivers{l}.pval;
end
end

end



% Build methylation associations.
% Each module for each methylation cluster.

% Negative associations.
methlclusmembers=zeros(1,nmethlvalidgenes);
for n=1:nmethlclus
methlclusmembers(methlclus{n})=n;
end
k=nmethlclus;
for n=1:nmethlvalidgenes
if (methlclusmembers(n)==0)
k=k+1; methlclusmembers(n)=k;
end
end

nmethlclus2=k;
clear methpassengers;
for n=1:nmethlclus2
methpassengers{n}=[];
end

isassociated=zeros(1,nmethlvalidgenes);
for n=1:ngenes
for i=1:jassociations{n}.ndrivers
if ((strcmp(jassociations{n}.drivers{i}.type,'methylation')==1)&(jassociations{n}.drivers{i}.dir==-1))
k=jassociations{n}.drivers{i}.ind;
val=new_missing_corrcoef(transpose([jmethlvaliddata(k,:);jmrnadata(n,:)]));
val=val(1,2);
if ((val<=-0.3)&(methl2TF(k)>0))
l=methlclusmembers(k); isassociated(k)=1;
methpassengers{l}=union(methpassengers{l},n);
end
end
end
end

for methind=1:nmethlclus2
if (length(methpassengers{methind})>=10)
nmodules=nmodules+1;
modules{nmodules}.drivertype='methylation';
modules{nmodules}.chr=0;
modules{nmodules}.driverinds(1)=methind;
modules{nmodules}.drivers{1}='';
modules{nmodules}.direction=-1;
subset=intersect(find(methlclusmembers==methind),find(isassociated==1));
modules{nmodules}.nregs=length(subset);
modules{nmodules}.reginds=subset;
modules{nmodules}.regs={};
for i=1:length(subset)
j=subset(i); modules{nmodules}.regs{i}=methlvalidgenes{j}.genename;
end
modules{nmodules}.npassengers=length(methpassengers{methind});
modules{nmodules}.passengerinds=methpassengers{methind};
modules{nmodules}.passengers={};
modules{nmodules}.lls=[]; modules{nmodules}.pvals=[];
for i=1:modules{nmodules}.npassengers
j=modules{nmodules}.passengerinds(i);
modules{nmodules}.passengers{i}=genes{j}.genename;
k=1; l=0;
while ((k<=jassociations{j}.ndrivers)&(l==0))
if (strcmp(jassociations{j}.drivers{k}.type,'methylation')==1)
m=jassociations{j}.drivers{k}.ind;
if (methlclusmembers(m)==methind)
l=k;
end
end
k=k+1;
end
modules{nmodules}.lls(i)=jassociations{j}.drivers{l}.ll;
modules{nmodules}.pvals(i)=jassociations{j}.drivers{l}.pval;
end
end
end


% Build mir association modules.
% Each module for each mir cluster.

% Negative associations.
mirclusmembers=zeros(1,nmirs);
for n=1:nmirclus
mirclusmembers(mirclus{n})=n;
end
k=nmirclus;
for n=1:nmirs
if (mirclusmembers(n)==0)
k=k+1; mirclusmembers(n)=k;
end
end

nmirclus2=k;
clear mirpassengers;
for n=1:nmirclus2
mirpassengers{n}=[];
end

ismirassociated=zeros(1,nmirs);
for n=1:ngenes
for i=1:jassociations{n}.ndrivers
if ((strcmp(jassociations{n}.drivers{i}.type,'mir')==1)&(jassociations{n}.drivers{i}.dir==-1))
k=jassociations{n}.drivers{i}.ind;
l=mirclusmembers(k); ismirassociated(k)=1;
mirpassengers{l}=union(mirpassengers{l},n);
end
end
end

for mirind=1:nmirclus2
if (length(mirpassengers{mirind})>=10)
nmodules=nmodules+1;
modules{nmodules}.drivertype='mir';
modules{nmodules}.chr=0;
modules{nmodules}.driverinds(1)=mirind;
modules{nmodules}.drivers{1}='';
modules{nmodules}.direction=-1;
subset=intersect(find(mirclusmembers==mirind),find(ismirassociated==1));
modules{nmodules}.nregs=length(subset);
modules{nmodules}.reginds=subset;
modules{nmodules}.regs={};
for i=1:length(subset)
j=subset(i); modules{nmodules}.regs{i}=mirs{j}.genename;
end
modules{nmodules}.npassengers=length(mirpassengers{mirind});
modules{nmodules}.passengerinds=mirpassengers{mirind};
modules{nmodules}.passengers={};
modules{nmodules}.lls=[]; modules{nmodules}.pvals=[];
for i=1:modules{nmodules}.npassengers
j=modules{nmodules}.passengerinds(i);
modules{nmodules}.passengers{i}=genes{j}.genename;
k=1; l=0;
while ((k<=jassociations{j}.ndrivers)&(l==0))
if (strcmp(jassociations{j}.drivers{k}.type,'mir')==1)
m=jassociations{j}.drivers{k}.ind;
if (mirclusmembers(m)==mirind)
l=k;
end
end
k=k+1;
end
modules{nmodules}.lls(i)=jassociations{j}.drivers{l}.ll;
modules{nmodules}.pvals(i)=jassociations{j}.drivers{l}.pval;
end
end
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Merge mRNA association modules to module sets.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Check the overlap of modules.
modintersect=zeros(nmodules,nmodules);
for n=1:(nmodules-1)
for m=(n+1):nmodules
modintersect(n,m)=length(intersect(modules{n}.passengerinds,modules{m}.passengerinds));
end
end

% Merge modules.
% The intersection of two modules >= 1/3 of the smallest size.
% Do not merge modules with intra segment CNVs.
% Do not merge module pairs of inter segment CNVs.
mergeable=zeros(nmodules,nmodules);
for n=1:(nmodules-1)
for m=(n+1):nmodules
if ((strcmp(modules{n}.drivertype,'intra segment CNV')~=1)&(strcmp(modules{m}.drivertype,'intra segment CNV')~=1)&(~((strcmp(modules{n}.drivertype,'inter segment CNV')==1)&(strcmp(modules{m}.drivertype,'inter segment CNV')==1))))
n1=modules{n}.npassengers;
n2=modules{m}.npassengers;
n3=modintersect(n,m);
if ((n3>=(n1/3))|(n3>=(n2/3)))
mergeable(n,m)=1; mergeable(m,n)=1;
elseif (n3>=50)
mergeable(n,m)=1; mergeable(m,n)=1;
end
end
end
end

nexpairs=0; expairs=[];
for n=1:(nmodules-1)
for m=(n+1):nmodules
if ((strcmp(modules{n}.drivertype,'inter segment CNV')==1)&(strcmp(modules{m}.drivertype,'inter segment CNV')==1))
nexpairs=nexpairs+1; expairs(nexpairs,:)=[n m];
end
end
end

[ncomps,comps]=find_conn_comps2(nmodules,mergeable,nexpairs,expairs);

nmodulesets=ncomps; clear modulesets;
for n=1:ncomps
modulesets{n}=comps{n}.comps;
end



% Save mRNA module information.
save level1234_modules.mat nmodules modules nmodulesets modulesets jassociations;



