% Merge two or multiple datasets of the same type.
% Each dataset already merges the info of probes.
% Inputs: cdf data of genes from each dataset.
% Outputs: merged cdf dataset.
% Average the cdf values from multiple datasets.
% If there is only one valid dataset, then use the valid dataset to represent the values.
% Assume the number of columns for each dataset is the same, and the columns correspond to the same samples.

function [ngenes, genes, mergeddata] = merge_datasets(ndatasets, datasets)


% Find the union of genes in each dataset.  Also obtain the mapping of the gene to the index in each dataset.
% Choose the dataset of the highest number of genes as the start.  Then append additional genes from other datasets.
maxind=0; maxval=0;
for n=1:ndatasets
if (datasets{n}.ngenes>maxval)
maxind=n; maxval=datasets{n}.ngenes;
end
end
ngenes=0; clear genes geneinds; nsamples=datasets{1}.nsamples;
n=maxind;
for i=1:datasets{n}.ngenes
name=datasets{n}.genes{i}.name;
chr=datasets{n}.genes{i}.chr;
genes{i}.name=name;
genes{i}.chr=chr;
genes{i}.start=datasets{n}.genes{i}.start;
genes{i}.end=datasets{n}.genes{i}.end;
geneinds(i,:)=zeros(1,ndatasets);
geneinds(i,n)=i;
end
ngenes=datasets{n}.ngenes;

% Sort the gene names.
clear sortedgenes sortedgeneinds;
for i=1:ngenes
sortedgenes{i}=genes{i}.name;
end
[sortedgenes,sortedgeneinds]=sort(sortedgenes);

% Consider genes from other datasets.
for n=1:ndatasets
if (n~=maxind)
for i=1:datasets{n}.ngenes
name=datasets{n}.genes{i}.name;
chr=datasets{n}.genes{i}.chr;
k=find_str_list(sortedgenes,name);

% A new gene.
if (length(k)<=0)
ngenes=ngenes+1;
genes{ngenes}.name=name;
genes{ngenes}.chr=chr;
genes{ngenes}.start=datasets{n}.genes{i}.start;
genes{ngenes}.end=datasets{n}.genes{i}.end;
geneinds(ngenes,:)=zeros(1,ndatasets);
geneinds(ngenes,n)=i;
inds=find_insert_pos(sortedgenes,name);
if (inds(1)==0)
for j=(ngenes-1):-1:1
sortedgenes{j+1}=sortedgenes{j};
end
sortedgenes{1}=name;
sortedgeneinds(2:ngenes)=sortedgeneinds(1:(ngenes-1));
sortedgeneinds(1)=ngenes;
elseif (inds(2)==ngenes)
sortedgenes{ngenes}=name;
sortedgeneinds(ngenes)=ngenes;
else
for j=(ngenes-1):-1:inds(2)
sortedgenes{j+1}=sortedgenes{j};
end
sortedgenes{inds(2)}=name;
sortedgeneinds((inds(2)+1):ngenes)=sortedgeneinds(inds(2):(ngenes-1));
sortedgeneinds(inds(2))=ngenes;
end

% An existing gene.
else
k=k(1); k=sortedgeneinds(k);
genes{k}.start=min(genes{k}.start,datasets{n}.genes{i}.start);
genes{k}.end=max(genes{k}.end,datasets{n}.genes{i}.end);
geneinds(k,n)=i;
end

end
end
end



% Merge the data.
mergeddata=NaN*ones(ngenes,nsamples);
for n=1:ngenes
subset=geneinds(n,:); clear subdata;
subdata=NaN*ones(ndatasets,nsamples);
for j=1:ndatasets
if (subset(j)>0)
subdata(j,:)=datasets{j}.data(subset(j),:);
end
end
mask=((subdata>=0)|(subdata<0));
nvs=sum(mask);
inds=find(nvs>0);
tmp=subdata(:,inds).*mask(:,inds);
tmp(find(~((tmp>=0)|(tmp<0))))=0;
mergeddata(n,inds)=sum(tmp)./nvs(inds);
end


