% Merge the data of multiple probes of the same gene.
% Inputs: cdf data from multiple probes.
% Output: merged data of single gene.
% Difference from merge_probes_singlegenes: use the normalized fractional counts as P(x=-1),P(x=0),P(x=1) for each data point.
% For each sample, treat the data from each probe as an instantiation of the data from a gene.  Compute the average fractional counts of P(x=-1), P(x=0), P(x=1) for each gene.  Convert the average fractional counts into the cdf value.
% The input data are cdf values.
% Difference from merge_probes_singlegenes2: adjust the cdf values by the rescaling function.

function [ngenes, genes, mergeddata] = merge_probes_singlegenes3(nprobes, probes, nsamples, samples, data, ga)

% Find the assignment of probes to genes.
% Assume probes are sorted by chromosomal coordinates.

% The sorted probes can start with chromosome 0, meaning the location is unknown.
% Assume the chromosome numbers range in [1 24].

ngenes=0; clear genes chrstarts; chrstarts(1)=1;

for n=1:nprobes

str=probes{n}.genename;
i=1; j=0; 

k=probes{n}.chr; 
if (k>(length(chrstarts)))
i=ngenes+1;
else
i=chrstarts(k);
end


while ((i<=ngenes)&(j==0))

if ((strcmp(str,genes{i}.name)==1)&(probes{n}.chr<=genes{i}.chr))

j=i;
end
i=i+1;
end
if (j==0)
ngenes=ngenes+1;
genes{ngenes}.name=str;
genes{ngenes}.chr=probes{n}.chr;
genes{ngenes}.nprobes=1;
genes{ngenes}.probes(1)=n;

if ((ngenes>1)&(genes{ngenes}.chr>genes{ngenes-1}.chr))
l=genes{ngenes}.chr; chrstarts(l)=ngenes;

end

else
genes{j}.nprobes=genes{j}.nprobes+1;
genes{j}.probes(genes{j}.nprobes)=n;
end
end

for n=1:ngenes
c1s=zeros(1,genes{n}.nprobes); c2s=zeros(1,genes{n}.nprobes);
for i=1:genes{n}.nprobes
j=genes{n}.probes(i);
c1s(i)=probes{j}.start; c2s(i)=probes{j}.end;
end
genes{n}.chr=probes{genes{n}.probes(1)}.chr;
genes{n}.start=min([c1s c2s]);
genes{n}.end=max([c1s c2s]);
end

% For each gene on each sample, perform data merge.
mergeddata=zeros(ngenes,nsamples);

for n=1:ngenes

for m=1:nsamples

% Ignore the NaN values.
subset=genes{n}.probes;
subset=subset(find((data(subset,m)>=0)|(data(subset,m)<0)));

% If all probes are NaN on the sample, then assign the gene value to NaN.
if (length(subset)<=0)
mergeddata(n,m)=NaN;

else
subdata=data(subset,m);

% Calculate the probability vectors for each entry.
qdata=zeros(length(subset),1,3);
set1=find(~((subdata>=0)|(subdata<0)));
set2=find(subdata<=0);
set3=find(subdata>=1);
set4=find((subdata>0)&(subdata<1));
tmpmat=zeros(size(subdata));
tmpmat(set2)=1; qdata(:,:,1)=qdata(:,:,1)+tmpmat;
tmpmat=zeros(size(subdata));
tmpmat(set3)=1; qdata(:,:,3)=qdata(:,:,3)+tmpmat;
tmpmat=zeros(size(subdata));
tmpmat(set4)=((1-subdata(set4)).^ga)./(1-log((1-subdata(set4)).^ga));
qdata(:,:,1)=qdata(:,:,1)+tmpmat;
tmpmat=zeros(size(subdata));
tmpmat(set4)=((subdata(set4)).^ga)./(1-log((subdata(set4)).^ga));
qdata(:,:,3)=qdata(:,:,3)+tmpmat;
set0=union(set2,union(set3,set4));
q1=qdata(:,:,1); q2=zeros(size(subdata)); q3=qdata(:,:,3); 
q2(set0)=1-q1(set0)-q3(set0);
qdata(:,:,2)=qdata(:,:,2)+q2;

ps=zeros(1,3);
for i=1:3
ps(i)=sum(qdata(:,1,i));
end
ps=ps/length(subset);
val1=inv_prob_quant(ps(1),1,2);
val3=inv_prob_quant(ps(3),1,1);

mergeddata(n,m)=(val1+val3)/2;

%mergeddata(n,m)=mean(subdata);

end
end
end


