% Merge the segments generated by two CNV datasets.
% The segment info from each dataset includes the small recurrent fragments and large infrequent fragments.
% For small recurrent fragments, extract the consensus from the two datasets.
% For large infrequent fragments, apply refined partitions from the two datasets.
% Also generate the cdf value of the segments.

function [nsegments, segments, segmentCNVcdfs] = merge_segmentinfo(nsamples, ncnvprobes1, cnvprobes1, cnvdata1, cnvcdfs1, segmentinfo1, smallfrags1, ga1, ncnvprobes2, cnvprobes2, cnvdata2, cnvcdfs2, segmentinfo2, smallfrags2, ga2, distthre)


% Extract coordinates of cnv probes.
probecoords1=zeros(ncnvprobes1,3);
for i=1:ncnvprobes1
probecoords1(i,:)=[cnvprobes1{i}.chr cnvprobes1{i}.start cnvprobes1{i}.end];
end
probecoords2=zeros(ncnvprobes2,3);
for i=1:ncnvprobes2
probecoords2(i,:)=[cnvprobes2{i}.chr cnvprobes2{i}.start cnvprobes2{i}.end];
end


% Find the consensus of small fragments between two CNV datasets.
nfrags1=0; frags1=[];
for n=1:24
for i=1:smallfrags1{n}.nfrags
coordrange=[-1 -1];
coordrange(1)=smallfrags1{n}.frags{i}(1);
coordrange(2)=smallfrags1{n}.frags{i}(length(smallfrags1{n}.frags{i}));
nfrags1=nfrags1+1;
frags1(nfrags1,:)=[n coordrange cnvprobes1{coordrange(1)}.start cnvprobes1{coordrange(2)}.end];
end
end

nfrags2=0; frags2=[];
for n=1:24
for i=1:smallfrags2{n}.nfrags
coordrange=[-1 -1];
coordrange(1)=smallfrags2{n}.frags{i}(1);
coordrange(2)=smallfrags2{n}.frags{i}(length(smallfrags2{n}.frags{i}));
nfrags2=nfrags2+1;
frags2(nfrags2,:)=[n coordrange cnvprobes2{coordrange(1)}.start cnvprobes2{coordrange(2)}.end];
end
end

% Find the mapping between the fragments of the two datasets.
G=zeros(nfrags1,nfrags2);
for n=1:nfrags1
for i=1:nfrags2
if (frags1(n,1)==frags2(i,1))
if (length(intervaloverlap(frags1(n,4:5),frags2(i,4:5)))>0)
G(n,i)=1;
end
end
end
end

% Find the connected components of the bipartite graph.
[nc,cs]=find_bipartite_conn_comps(nfrags1,nfrags2,G);

% Generate overlap fragments.
noverlaps=nc; overlaps=[];
for n=1:noverlaps
if ((cs{n}.n1>0)&(cs{n}.n2>0))
a=min(min(frags1(cs{n}.comps1,4)),min(frags2(cs{n}.comps2,4)));
b=max(max(frags1(cs{n}.comps1,5)),max(frags2(cs{n}.comps2,5)));
overlaps(n,:)=[frags1(cs{n}.comps1(1),1) a b];
end
end
[YY,II]=sort(overlaps(:,1));
overlaps=overlaps(II,:);
for n=1:24
s=find(overlaps(:,1)==n);
u=overlaps(s,2);
[YY2,II2]=sort(u);
overlaps(s,:)=overlaps(s(II2),:);
end

% Remove the overlapped fragments with inconsistent CNV data.
removed=zeros(1,noverlaps);
for n=1:noverlaps
k=overlaps(n,1);
s1=find((frags1(:,1)==k)&(frags1(:,4)>=overlaps(n,2))&(frags1(:,5)<=overlaps(n,3)));
s2=find((frags2(:,1)==k)&(frags2(:,4)>=overlaps(n,2))&(frags2(:,5)<=overlaps(n,3)));
t1=[];
for i=1:length(s1)
t1=union(t1,frags1(s1(i),2):frags1(s1(i),3));
end
t2=[];
for i=1:length(s2)
t2=union(t2,frags2(s2(i),2):frags2(s2(i),3));
end
vals=new_missing_corrcoef(transpose([cnvdata1(t1,:);cnvdata2(t2,:)]));
val=mean(mean(vals(1:length(t1),(length(t1)+1):(length(t1)+length(t2)))));
if (val<0.3)
removed(n)=0;
end
end

% Remove the overlapped fragments if they are corelated with surrounded probes.
bds1=zeros(24,2);
bds1(1,1)=1; bds1(24,2)=ncnvprobes1;
for n=2:ncnvprobes1
if (cnvprobes1{n}.chr~=cnvprobes1{n-1}.chr)
bds1(cnvprobes1{n}.chr,1)=n;
bds1(cnvprobes1{n-1}.chr,2)=n-1;
end
end
bds2=zeros(24,2);
bds2(1,1)=1; bds2(24,2)=ncnvprobes2;
for n=2:ncnvprobes2
if (cnvprobes2{n}.chr~=cnvprobes2{n-1}.chr)
bds2(cnvprobes2{n}.chr,1)=n;
bds2(cnvprobes2{n-1}.chr,2)=n-1;
end
end

for n=1:noverlaps
if (removed(n)==0)
k=overlaps(n,1);
s1=find((frags1(:,1)==k)&(frags1(:,4)>=overlaps(n,2))&(frags1(:,5)<=overlaps(n,3)));
s2=find((frags2(:,1)==k)&(frags2(:,4)>=overlaps(n,2))&(frags2(:,5)<=overlaps(n,3)));
t1=[];
for i=1:length(s1)
t1=union(t1,frags1(s1(i),2):frags1(s1(i),3));
end
t2=[];
for i=1:length(s2)
t2=union(t2,frags2(s2(i),2):frags2(s2(i),3));
end
u1=max(bds1(k,1),min(t1)-100);
u2=min(bds1(k,2),max(t1)+100);
vals=new_missing_corrcoef(transpose([cnvdata1([u1 u2 t1],:)]));
val1=mean(mean(vals(1:2,3:(length(t1)+2))));
u1=max(bds2(k,1),min(t2)-100);
u2=min(bds2(k,2),max(t2)+100);
vals=new_missing_corrcoef(transpose([cnvdata2([u1 u2 t2],:)]));
val2=mean(mean(vals(1:2,3:(length(t2)+2))));
if ((val1>=0.3)&(val2>=0.3))
removed(n)=1;
end
end
end

% Remove the overlapped fragments if they cover only 1 position.
for n=1:noverlaps
if (overlaps(n,2)==overlaps(n,3))
removed(n)=1;
end
end


overlaps=overlaps(find(removed==0),:);
noverlaps=length(overlaps);


% For each sample, find the consensus boundaries from the two datasets.
clear consensusbds1 consensusbds2;
for n=1:nsamples
for i=1:24
consensusbds1{n}{i}=[]; consensusbds2{n}{i}=[]; 
end
end

for n=1:nsamples
for i=1:24
tmpbds1=[];
for j=1:segmentinfo1{n}.nsegments
k=segmentinfo1{n}.segments{j}.chr;
if (k==i)
tmpbds1=union(tmpbds1,segmentinfo1{n}.segments{j}.startprobe);
tmpbds1=union(tmpbds1,segmentinfo1{n}.segments{j}.endprobe);
end
end
tmpbds2=[];
for j=1:segmentinfo2{n}.nsegments
k=segmentinfo2{n}.segments{j}.chr;
if (k==i)
tmpbds2=union(tmpbds2,segmentinfo2{n}.segments{j}.startprobe);
tmpbds2=union(tmpbds2,segmentinfo2{n}.segments{j}.endprobe);
end
end

% Collapse adjacent boundaries.
chain1=zeros(1,length(tmpbds1));
for j=1:(length(tmpbds1)-1)
j1=tmpbds1(j); j2=tmpbds1(j+1);
if ((j2-j1)<=distthre)
chain1(j)=1;
end
end
j=1; vs=zeros(1,length(tmpbds1)); vs(length(tmpbds1))=1; frag=[];
while (j<=length(tmpbds1))
if ((j==1)|((j>1)&(chain1(j-1)==0)))
if (length(frag)>0)
m=floor(mean(frag));
vs(m)=1;
end
frag=j;
else
frag=union(frag,j);
end
j=j+1;
end
tmpbds1=tmpbds1(find(vs==1));

chain2=zeros(1,length(tmpbds2));
for j=1:(length(tmpbds2)-1)
j1=tmpbds2(j); j2=tmpbds2(j+1);
if ((j2-j1)<=distthre)
chain2(j)=1;
end
end
j=1; vs=zeros(1,length(tmpbds2)); vs(length(tmpbds2))=1; frag=[];
while (j<=length(tmpbds2))
if ((j==1)|((j>1)&(chain2(j-1)==0)))
if (length(frag)>0)
m=floor(mean(frag));
vs(m)=1;
end
frag=j;
else
frag=union(frag,j);
end
j=j+1;
end
tmpbds2=tmpbds2(find(vs==1));

% Find consensus boundaries.
% For a boundary in cnv1 find the boundaries in cnv2 that are within distthre probes in cnv2.
% For a boundary in cnv2 find the boundaries in cnv1 that are within distthre probes in cnv1.
% Build a bipartite graph of boundaries of the two cnv datasets.
% For each connected component find representative boundaries.
G=zeros(length(tmpbds1),length(tmpbds2));
for j=1:length(tmpbds1)
k=tmpbds1(j);
a=max(find((probecoords2(:,1)==i)&(probecoords2(:,2)<=probecoords1(k,2))));
for l=1:length(tmpbds2)
if (abs(tmpbds2(l)-a)<=distthre)
G(j,l)=1;
end
end
end
for j=1:length(tmpbds2)
k=tmpbds2(j);
a=max(find((probecoords1(:,1)==i)&(probecoords1(:,2)<=probecoords2(k,2))));
for l=1:length(tmpbds1)
if (abs(tmpbds1(l)-a)<=distthre)
G(l,j)=1;
end
end
end
[nc,cs]=find_bipartite_conn_comps(length(tmpbds1),length(tmpbds2),G);

for j=1:nc
if ((cs{j}.n1>0)&(cs{j}.n2>0))
a=tmpbds1(cs{j}.comps1(1));
consensusbds1{n}{i}=union(consensusbds1{n}{i},a);
a=tmpbds2(cs{j}.comps2(1));
consensusbds2{n}{i}=union(consensusbds2{n}{i},a);
end
end

% If cnvdata2 is not valid on that sample, then the consensus boundaries are the boundaries in cnvdata1.  Vice versa for cnvdata2.
tmp2=cnvdata2(tmpbds2(1):tmpbds2(length(tmpbds2)),n);
if (sum(~((tmp2>=0)|(tmp2<0)))>=(length(tmp2)/2))
consensusbds1{n}{i}=tmpbds1;
consensusbds2{n}{i}(1)=min(find(probecoords2(:,1)==i));
consensusbds2{n}{i}(length(tmpbds1))=max(find(probecoords2(:,1)==i));
for j=2:(length(tmpbds1)-1)
k=tmpbds1(j);
a=max(find((probecoords2(:,1)==i)&(probecoords2(:,2)<=probecoords1(k,2))));
consensusbds2{n}{i}(j)=a;
end
end
tmp1=cnvdata1(tmpbds1(1):tmpbds1(length(tmpbds1)),n);
if (sum(~((tmp1>=0)|(tmp1<0)))>=(length(tmp1)/2))
consensusbds2{n}{i}=tmpbds2;
consensusbds1{n}{i}(1)=min(find(probecoords1(:,1)==i));
consensusbds1{n}{i}(length(tmpbds2))=max(find(probecoords1(:,1)==i));
for j=2:(length(tmpbds2)-1)
k=tmpbds2(j);
a=max(find((probecoords1(:,1)==i)&(probecoords1(:,2)<=probecoords2(k,2))));
consensusbds1{n}{i}(j)=a;
end
end
if ((sum(~((tmp1>=0)|(tmp1<0)))>=(length(tmp1)/2))&(sum(~((tmp2>=0)|(tmp2<0)))>=(length(tmp2)/2)))
consensusbds1{n}{i}=tmpbds1;
consensusbds2{n}{i}=tmpbds2;
end

% end of for i=1:24.
end

% end of for n=1:nsamples
end


% Combine the boundaries from all samples.
clear probebds1;
for n=1:24
probebds1{n}=[];
end
for n=1:nsamples
for i=1:24
probebds1{i}=union(probebds1{i},consensusbds1{n}{i});
end
end

% Collapse adjacent boundaries.
for n=1:24
chain1=zeros(1,length(probebds1{n}));
for j=1:(length(probebds1{n})-1)
j1=probebds1{n}(j); j2=probebds1{n}(j+1);
if ((j2-j1)<=distthre)
chain1(j)=1;
end
end
j=1; vs=zeros(1,length(probebds1{n})); vs(length(probebds1{n}))=1; frag=[];
while (j<=length(probebds1{n}))
if ((j==1)|((j>1)&(chain1(j-1)==0)))
if (length(frag)>0)
m=floor(mean(frag));
vs(m)=1;
end
frag=j;
else
frag=union(frag,j);
end
j=j+1;
end
probebds1{n}=probebds1{n}(find(vs==1));
end

clear probebds2;
for n=1:24
probebds2{n}=[];
end
for n=1:nsamples
for i=1:24
probebds2{i}=union(probebds2{i},consensusbds2{n}{i});
end
end

for n=1:24
chain2=zeros(1,length(probebds2{n}));
for j=1:(length(probebds2{n})-1)
j1=probebds2{n}(j); j2=probebds2{n}(j+1);
if ((j2-j1)<=distthre)
chain2(j)=1;
end
end
j=1; vs=zeros(1,length(probebds2{n})); vs(length(probebds2{n}))=1; frag=[];
while (j<=length(probebds2{n}))
if ((j==1)|((j>1)&(chain2(j-1)==0)))
if (length(frag)>0)
m=floor(mean(frag));
vs(m)=1;
end
frag=j;
else
frag=union(frag,j);
end
j=j+1;
end
probebds2{n}=probebds2{n}(find(vs==1));
end

% Find the consensus boundaries between cnv1 and cnv2 again.
% For a boundary in cnv1 find the boundaries in cnv2 that are within distthre probes in cnv2.
% For a boundary in cnv2 find the boundaries in cnv1 that are within distthre probes in cnv1.
% Build a bipartite graph of boundaries of the two cnv datasets.
% For each connected component find representative boundaries.
clear consensusprobebds1 consensusprobebds2;
for n=1:24
consensusprobebds1{n}=[]; consensusprobebds2{n}=[];
end
for n=1:24
tmpbds1=probebds1{n}; tmpbds2=probebds2{n}; 
G=zeros(length(tmpbds1),length(tmpbds2));
for j=1:length(tmpbds1)
k=tmpbds1(j);
a=max(find((probecoords2(:,1)==n)&(probecoords2(:,2)<=probecoords1(k,2))));
for l=1:length(tmpbds2)
if (abs(tmpbds2(l)-a)<=distthre)
G(j,l)=1;
end
end
end
for j=1:length(tmpbds2)
k=tmpbds2(j);
a=max(find((probecoords1(:,1)==n)&(probecoords1(:,2)<=probecoords2(k,2))));
for l=1:length(tmpbds1)
if (abs(tmpbds1(l)-a)<=distthre)
G(l,j)=1;
end
end
end
[nc,cs]=find_bipartite_conn_comps(length(tmpbds1),length(tmpbds2),G);

for j=1:nc
if ((cs{j}.n1>0)&(cs{j}.n2>0))
a=tmpbds1(cs{j}.comps1(1));
consensusprobebds1{n}=union(consensusprobebds1{n},a);
a=tmpbds2(cs{j}.comps2(1));
consensusprobebds2{n}=union(consensusprobebds2{n},a);
end
end

% If cnvdata2 is not valid on that sample, then the consensus boundaries are the boundaries in cnvdata1.  Vice versa for cnvdata2.
tmp2=cnvdata2(tmpbds2(1):tmpbds2(length(tmpbds2)),n);
if (sum(~((tmp2>=0)|(tmp2<0)))>=(length(tmp2)/2))
consensusprobebds1{n}=tmpbds1;
consensusprobebds2{n}(1)=min(find(probecoords2(:,1)==n));
consensusprobebds2{n}(length(tmpbds1))=max(find(probecoords2(:,1)==n));
for j=2:(length(tmpbds1)-1)
k=tmpbds1(j);
a=max(find((probecoords2(:,1)==n)&(probecoords2(:,2)<=probecoords1(k,2))));
consensusprobebds2{n}(j)=a;
end
end
tmp1=cnvdata1(tmpbds1(1):tmpbds1(length(tmpbds1)),n);
if (sum(~((tmp1>=0)|(tmp1<0)))>=(length(tmp1)/2))
consensusprobebds2{n}=tmpbds2;
consensusprobebds1{n}(1)=min(find(probecoords1(:,1)==n));
consensusprobebds1{n}(length(tmpbds2))=max(find(probecoords1(:,1)==n));
for j=2:(length(tmpbds2)-1)
k=tmpbds2(j);
a=max(find((probecoords1(:,1)==n)&(probecoords1(:,2)<=probecoords2(k,2))));
consensusprobebds1{n}(j)=a;
end
end
if ((sum(~((tmp1>=0)|(tmp1<0)))>=(length(tmp1)/2))&(sum(~((tmp2>=0)|(tmp2<0)))>=(length(tmp2)/2)))
consensusprobebds1{n}=tmpbds1;
consensusprobebds2{n}=tmpbds2;
end

% end of for n=1:24.
end


% Express the boundaries in a matrix.
nn=0; segmat=[];
for n=1:24
for i=1:length(consensusprobebds1{n})
c1=cnvprobes1{consensusprobebds1{n}(i)}.start;
c2=cnvprobes1{consensusprobebds1{n}(i)}.end;
c3=cnvprobes2{consensusprobebds2{n}(i)}.start;
c4=cnvprobes2{consensusprobebds2{n}(i)}.end;
d1=min([c1 c2 c3 c4]); d2=max([c1 c2 c3 c4]);
nn=nn+1; 
segmat(nn,:)=[n d1 d2];
end
end


% Do not need this part.
newnn=nn; newsegmat=segmat;


% Build joint segments.
nsegments=0; clear segments;
for n=1:(newnn-1)
% Skip if the current boundary is the last boundary on a chromosome.
if (newsegmat(n,1)==newsegmat(n+1,1))
nsegments=nsegments+1;
segments{nsegments}.chr=newsegmat(n,1);
c1=newsegmat(n,2); c2=newsegmat(n+1,3);
a=min(find((probecoords1(:,1)==newsegmat(n,1))&(probecoords1(:,3)>=c1)));
segments{nsegments}.startprobe1=a;
a=max(find((probecoords1(:,1)==newsegmat(n,1))&(probecoords1(:,2)<=c2)));
segments{nsegments}.endprobe1=a;
a=min(find((probecoords2(:,1)==newsegmat(n,1))&(probecoords2(:,3)>=c1)));
segments{nsegments}.startprobe2=a;
a=max(find((probecoords2(:,1)==newsegmat(n,1))&(probecoords2(:,2)<=c2)));
segments{nsegments}.endprobe2=a;
segments{nsegments}.startcoord=c1;
segments{nsegments}.endcoord=c2;
end
end


% Generate cdf values of each segment.
% For each segment and each sample, calculate the fractional counts of states +1, 0, -1.  Convert the fractional counts to the most fit cdf value using ga=1.
segmentCNVcdfs=NaN*ones(nsegments,nsamples);
for n=1:nsegments
n1=segments{n}.endprobe1-segments{n}.startprobe1+1;
subset1=segments{n}.startprobe1:segments{n}.endprobe1;
subcdfs1=cnvcdfs1(subset1,:);
qdata1=zeros(n1,nsamples,3);
set1=find(~((cnvcdfs1(subset1,:)>=0)|(cnvcdfs1(subset1,:)<0)));
set2=find(cnvcdfs1(subset1,:)<=0);
set3=find(cnvcdfs1(subset1,:)>=1);
set4=find((cnvcdfs1(subset1,:)>0)&(cnvcdfs1(subset1,:)<1));
tmpmat=zeros(size(cnvcdfs1(subset1,:)));
tmpmat(set2)=1; qdata1(:,:,1)=qdata1(:,:,1)+tmpmat;
tmpmat=zeros(size(cnvcdfs1(subset1,:)));
tmpmat(set3)=1; qdata1(:,:,3)=qdata1(:,:,3)+tmpmat;
tmpmat=zeros(size(cnvcdfs1(subset1,:)));
tmpmat(set4)=((1-subcdfs1(set4)).^ga1)./(1-log((1-subcdfs1(set4)).^ga1));
qdata1(:,:,1)=qdata1(:,:,1)+tmpmat;
tmpmat=zeros(size(cnvcdfs1(subset1,:)));
tmpmat(set4)=((subcdfs1(set4)).^ga1)./(1-log((subcdfs1(set4)).^ga1));
qdata1(:,:,3)=qdata1(:,:,3)+tmpmat;
set0=union(set2,union(set3,set4));
q1=qdata1(:,:,1); q2=zeros(size(cnvcdfs1(subset1,:))); q3=qdata1(:,:,3); 
q2(set0)=1-q1(set0)-q3(set0);
qdata1(:,:,2)=qdata1(:,:,2)+q2;

n2=segments{n}.endprobe2-segments{n}.startprobe2+1;
subset2=segments{n}.startprobe2:segments{n}.endprobe2;
subcdfs2=cnvcdfs2(subset2,:);
qdata2=zeros(n2,nsamples,3);
set1=find(~((cnvcdfs2(subset2,:)>=0)|(cnvcdfs2(subset2,:)<0)));
set2=find(cnvcdfs2(subset2,:)<=0);
set3=find(cnvcdfs2(subset2,:)>=1);
set4=find((cnvcdfs2(subset2,:)>0)&(cnvcdfs2(subset2,:)<1));
tmpmat=zeros(size(cnvcdfs2(subset2,:)));
tmpmat(set2)=1; qdata2(:,:,1)=qdata2(:,:,1)+tmpmat;
tmpmat=zeros(size(cnvcdfs2(subset2,:)));
tmpmat(set3)=1; qdata2(:,:,3)=qdata2(:,:,3)+tmpmat;
tmpmat=zeros(size(cnvcdfs2(subset2,:)));
tmpmat(set4)=((1-subcdfs2(set4)).^ga2)./(1-log((1-subcdfs2(set4)).^ga2));
qdata2(:,:,1)=qdata2(:,:,1)+tmpmat;
tmpmat=zeros(size(cnvcdfs2(subset2,:)));
tmpmat(set4)=((subcdfs2(set4)).^ga2)./(1-log((subcdfs2(set4)).^ga2));
qdata2(:,:,3)=qdata2(:,:,3)+tmpmat;
set0=union(set2,union(set3,set4));
q1=qdata2(:,:,1); q2=zeros(size(cnvcdfs2(subset2,:))); q3=qdata2(:,:,3); 
q2(set0)=1-q1(set0)-q3(set0);
qdata2(:,:,2)=qdata2(:,:,2)+q2;


% Calculate the fractional count from all valid probes.
for i=1:nsamples
p1s=zeros(1,3); p2s=zeros(1,3);
v1=find((cnvcdfs1(subset1,i)>=0)&(cnvcdfs1(subset1,i)<=1)); nv1=length(v1);
for j=1:3
p1s(j)=sum(qdata1(v1,i,j));
end
v2=find((cnvcdfs2(subset2,i)>=0)&(cnvcdfs2(subset2,i)<=1)); nv2=length(v2);
for j=1:3
p2s(j)=sum(qdata2(v2,i,j));
end
ps=zeros(1,3);
for j=1:3
ps(j)=p1s(j)+p2s(j);
end
if ((nv1+nv2)>0)
ps=ps/(nv1+nv2);
val1=inv_prob_quant(ps(1),1,2);
val3=inv_prob_quant(ps(3),1,1);
segmentCNVcdfs(n,i)=(val1+val3)/2;
end
end

end


