/* Utility functions of graphs.
 */

# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <math.h>
# include <time.h>
# include "motif.h"
# define MAXLINEWIDTH 1000000
# define NAMELENGTH 30
# define LONGNAMELENGTH 100
// # define LINEWIDTH 30
# define max(i,j) (i>j ? i : j)
# define min(i,j) (i<j ? i : j)
# define argmax(i,j) (j>i ? 2 : 1)

extern int pair_cmp(struct pair *p1, struct pair *p2);
extern int pair_cmp_inv(struct pair *p1, struct pair *p2);
extern int subseteq(int nset1, int *set1, int nset2, int *set2);
extern int inlist(int Nv, int *v, int target);
extern int *int_union(int nset1, int *set1, int nset2, int *set2, int *nset_pnt);
extern int isequal(int nset1, int *set1, int nset2, int *set2);

struct component * find_conn_comps(int nnodes, int *G, int *nconncomps_pnt);
void recurse_label(int ind, int nnodes, int *G, int *labels, int label);
struct component * find_cliques(int nsubnodes, int *subG, int *ncliques_pnt);



// Find connected components of a graph.
struct component * find_conn_comps(int nnodes, int *G, int *nconncomps_pnt)
{
  int i, j, k, l, m, n, length, *labels, flag, nconncomps;
  struct pair *nmempairs;
  struct component *conncomps, *newconncomps;
  
  labels=malloc(sizeof(int)*nnodes);
  for (i=0; i<=(nnodes-1); i++)
    *(labels+i)=-1;
  flag=1;

  // Label all the singletons.
  k=-1;
  for (i=0; i<=(nnodes-1); i++)
    {
      l=0;
      for (j=0; j<=(nnodes-1); j++)
	if ((j!=i)&&(*(G+i*nnodes+j)>0))
	  l++;
      if (l==0)
	{
	  k++; *(labels+i)=k;
	}
    }

  // Recursively label members in connected components.
  while (flag==1)
    {
      // Check if all nodes are labeled.
      n=0;
      for (i=0; i<=(nnodes-1); i++)
	if (*(labels+i)<0)
	  n++;

      // If yes, then return.
      if (n==0)
	flag=0;

      // Otherwise pick up an unlabeled node and recursively label its neighbors.
      else
	{
	  i=0; j=-1;
	  while ((i<nnodes)&&(j==-1))
	    {
	      if (*(labels+i)<0)
		j=i;
	      i++;
	    }
	  k++;
	  recurse_label(j,nnodes,G,labels,k);
	}
    }

  // Find the connected components from the labels.
  nconncomps=k+1; conncomps=malloc(sizeof(struct component)*nconncomps);
  nmempairs=malloc(sizeof(struct pair)*nconncomps);
  for (i=0; i<=(nconncomps-1); i++)
    {
      (conncomps+i)->nmembers=0;
      (conncomps+i)->members=malloc(sizeof(int));
    }

  for (i=0; i<=(nnodes-1); i++)
    {
      j=*(labels+i);
      (conncomps+j)->nmembers+=1;
      (conncomps+j)->members=realloc((conncomps+j)->members,sizeof(int)*(conncomps+j)->nmembers);
      *((conncomps+j)->members+(conncomps+j)->nmembers-1)=i;
      (nmempairs+j)->i=j; (nmempairs+j)->j=(conncomps+j)->nmembers;      
    }

  // Sort the connected components by size.
  // Discard null components.
  qsort(nmempairs,nconncomps,sizeof(struct pair),(* pair_cmp)); l=0;
  
  newconncomps=malloc(sizeof(struct component)*nconncomps);
  for (n=0; n<=(nconncomps-1); n++)
    {      
      i=(nmempairs+n)->i;
      if ((conncomps+i)->nmembers>0)
	{
	  (newconncomps+n)->nmembers=(conncomps+i)->nmembers;
	  (newconncomps+n)->members=malloc(sizeof(int)*(newconncomps+n)->nmembers);
	  for (j=0; j<=((newconncomps+n)->nmembers-1); j++)
	    *((newconncomps+n)->members+j)=*((conncomps+i)->members+j);
	  l++;
	}
    }
  for (n=0; n<=(nconncomps-1); n++)
    free((conncomps+n)->members);
  free(conncomps);
  nconncomps=l; free(nmempairs); free(labels);
    
  *nconncomps_pnt=nconncomps; 
  return newconncomps;
  
}


// Recursively label nodes in a graph.
void recurse_label(int ind, int nnodes, int *G, int *labels, int label)
{
  int i, j, k;

  // Label the target node.
  *(labels+ind)=label;

  // Recursively visit neighbors.
  for (i=0; i<=(nnodes-1); i++)    
    if ((i!=ind)&&(*(G+ind*nnodes+i)>0)&&(*(labels+i)<0))
      recurse_label(i,nnodes,G,labels,label);

  return;
}


// Find all maximal cliques of a graph.
struct component * find_cliques(int nsubnodes, int *subG, int *ncliques_pnt)
{
  int i, j, k, l, m, n, length, flag, ncls, *ismax, *startpos, *endpos, *isold, *pnt1, *pnt2, nmaxclsize, ncliques;
  struct component *cls, *cliques;

  
  // Start from nodes as cliques.
  ncls=nsubnodes; cls=malloc(sizeof(struct component)*nsubnodes);
  ismax=malloc(sizeof(int)*nsubnodes);
  for (i=0; i<=(nsubnodes-1); i++)
    {
      *(ismax+i)=1; (cls+i)->nmembers=1; 
      (cls+i)->members=malloc(sizeof(int));
      *((cls+i)->members+0)=i;
    }

  startpos=malloc(sizeof(int)*2); *(startpos+0)=0; *(startpos+1)=0;
  endpos=malloc(sizeof(int)*2); *(endpos+0)=0; *(endpos+1)=nsubnodes-1;
  nmaxclsize=1;

 
  // Merge cliques until no two cliques are mergeable.
  flag=1; isold=malloc(sizeof(int)*ncls);
  for (i=0; i<=(ncls-1); i++)
    *(isold+i)=0;  

  while (flag==1)
    {
      int nm, *list, ff;      
      int *newismax, *newisold;
      struct component *newcls;	     

      // Find all mergeable clique pairs.      
      nm=0; list=malloc(sizeof(int)*2);
      for (i=0; i<=(ncls-2); i++)
	{
	  for (j=(i+1); j<=(ncls-1); j++)
	    {
	      ff=1;
	     
	      // One old clique and one new clique or two new cliques.
	      if ((*(isold+i)+*(isold+j))>=2)
		ff=0;	      

	      // They are not subsumed to each other.
	      if (ff==1)
		{
		  if (subseteq((cls+i)->nmembers,(cls+i)->members,(cls+j)->nmembers,(cls+j)->members)==1)
		    {
		      ff=0; *(ismax+i)=0;
		    }
		  if (subseteq((cls+j)->nmembers,(cls+j)->members,(cls+i)->nmembers,(cls+i)->members)==1)
		    {
		      ff=0; *(ismax+j)=0;
		    }
		}
	     	      
	      // All members in clique i are connected to all members of clique j.
	      if (ff==1)
		{
		  int nset1, *set1;
		  set1=int_union((cls+i)->nmembers,(cls+i)->members,(cls+j)->nmembers,(cls+j)->members,&nset1);
		  m=0;
		  for (k=0; k<=(nset1-1); k++)
		    for (l=0; l<=(nset1-1); l++)
		      if (*(subG+*(set1+k)*nsubnodes+*(set1+l))>0)
			m++;
		 
		  if (m<(nset1*nset1))
		    ff=0;
		  free(set1);
		}
	
	      // The merged clique does not already appear.
	      if (ff==1)
		{
		  int nset1, *set1;
		  set1=int_union((cls+i)->nmembers,(cls+i)->members,(cls+j)->nmembers,(cls+j)->members,&nset1);
		  if (nmaxclsize>=nset1)
		    {
		      l=*(startpos+nset1);
		      while ((l<=*(endpos+nset1))&&(ff==1))
			{
			  if (isequal(nset1,set1,(cls+l)->nmembers,(cls+l)->members)==1)
			    ff=0;
			  l++;
			}
		    }
		  free(set1);
		}
	 
	      // The merged clique is not already generated in this run.
	      if (ff==1)
		{
		  int nset1, *set1, nset2, *set2;
		  set1=int_union((cls+i)->nmembers,(cls+i)->members,(cls+j)->nmembers,(cls+j)->members,&nset1);
		  k=0;
		  while ((k<nm)&&(ff==1))
		    {
		      set2=int_union((cls+*(list+k*2+0))->nmembers,(cls+*(list+k*2+0))->members,(cls+*(list+k*2+1))->nmembers,(cls+*(list+k*2+1))->members,&nset2);
		      if (isequal(nset1,set1,nset2,set2)==1)
			ff=0;
		      k++;
		      free(set2);
		    }
		  free(set1); 
		}	      	      	      	    

	      // Add the pair to the merge list.
	      if (ff==1)
		{
		  nm+=1; list=realloc(list,sizeof(int)*nm*2);
		  *(list+(nm-1)*2+0)=i; *(list+(nm-1)*2+1)=j;		 
		}

	    }
	}

      // Merge the pairs.
      if (nm>0)
	{
	  for (i=0; i<=(ncls-1); i++)
	    *(isold+i)=1;
	  ismax=realloc(ismax,sizeof(int)*(ncls+nm));
	  isold=realloc(isold,sizeof(int)*(ncls+nm));
	  for (k=0; k<=(nm-1); k++)
	    {
	      i=*(list+k*2+0); j=*(list+k*2+1); ncls++;
	      cls=realloc(cls,sizeof(struct component)*ncls);
	      (cls+ncls-1)->members=int_union((cls+i)->nmembers,(cls+i)->members,(cls+j)->nmembers,(cls+j)->members,&l);	      
	      (cls+ncls-1)->nmembers=l;
	      *(ismax+i)=0; *(ismax+j)=0; *(ismax+ncls-1)=1; *(isold+ncls-1)=0;
	    }

	  // Debug
	  printf("ncls=%d, nm=%d\n",ncls,nm);
	  
	}

      // No mergeable pairs: stop.
      else
	flag=0;


      // Sort cliques by size.
      if (flag==1)
	{	  
	  struct pair *ns;
	  
	  ns=malloc(sizeof(struct pair)*ncls);
	  for (i=0; i<=(ncls-1); i++)
	    {
	      (ns+i)->i=i;
	      (ns+i)->j=(cls+i)->nmembers;
	    }
	  
	  qsort(ns,ncls,sizeof(struct pair),(* pair_cmp_inv));
	  	  
	  newcls=malloc(sizeof(struct component)*ncls);
	  newismax=malloc(sizeof(int)*ncls);
	  newisold=malloc(sizeof(int)*ncls);
	  for (n=0; n<=(ncls-1); n++)
	    {
	      i=(ns+n)->i;
	      (newcls+n)->nmembers=(cls+i)->nmembers;
	      (newcls+n)->members=malloc(sizeof(int)*(newcls+n)->nmembers);
	      for (j=0; j<=((newcls+n)->nmembers-1); j++)
		*((newcls+n)->members+j)=*((cls+i)->members+j);
	      *(newismax+n)=*(ismax+i); *(newisold+n)=*(isold+i);
	    }
	  free(cls); free(ismax); free(isold);
	  cls=newcls; ismax=newismax; isold=newisold;
	  for (i=0; i<=(ncls-2); i++)
	    {
	      if ((cls+i)->nmembers<(cls+i+1)->nmembers)
		{
		  int k1, k2;
		  k1=(cls+i)->nmembers; k2=(cls+i+1)->nmembers;
		  if (nmaxclsize<max(k1,k2))
		    {				
		      nmaxclsize=max(k1,k2);
		      startpos=realloc(startpos,sizeof(int)*(nmaxclsize+1));
		      endpos=realloc(endpos,sizeof(int)*(nmaxclsize+1));
		      *(endpos+k1)=i; *(startpos+k2)=i+1;
		    }
		}
	    }
	  *(endpos+(cls+ncls-1)->nmembers)=ncls-1;
	  free(ns);

	}	  
    }

  // Report maximal cliques.
  ncliques=0; cliques=malloc(sizeof(struct component));
  for (i=0; i<=(ncls-1); i++)
    {
      if ((*(ismax+i)==1)&&((cls+i)->nmembers>1))
	{
	  ncliques++; cliques=realloc(cliques,sizeof(struct component)*ncliques);
	  (cliques+ncliques-1)->nmembers=(cls+i)->nmembers;
	  (cliques+ncliques-1)->members=malloc(sizeof(int)*(cliques+ncliques-1)->nmembers);
	  for (j=0; j<=((cliques+ncliques-1)->nmembers-1); j++)
	    *((cliques+ncliques-1)->members+j)=*((cls+i)->members+j);
	}
    }

  // Debug
  printf("ncls=%d, nmaxcls=%d\n",ncls,ncliques);

  for (i=0; i<=(ncls-1); i++)
    free((cls+i)->members);
  free(cls); free(ismax); free(isold); free(startpos); free(endpos); 

  *ncliques_pnt=ncliques; 

  return cliques;
 
}
