% Recursively label nodes in a graph.
% Difference from recurse_label.m: not on a bipartite graph.
% Difference from recurse_label2.m: avoid labeling expairs in the same components.

function labeled = recurse_label3(ind, nnodes, G, labeled, label, nexpairs, expairs)

% Label the target node.
labeled(ind)=label;

% Find the neighbors of node ind.
neighbors=find(G(ind,:)>0); neighbors=setdiff(neighbors,ind);

% Exclude the nodes that form expairs with ind.
exneighbors=[];
tmp=find(expairs(:,1)==ind);
if (length(tmp)>0)
exneighbors=union(exneighbors,expairs(tmp,2));
end
tmp=find(expairs(:,2)==ind);
if (length(tmp)>0)
exneighbors=union(exneighbors,expairs(tmp,1));
end
exneighbors=transpose(exneighbors); exneighbors=intersect(neighbors,exneighbors);

% If some expairs are both in the neighbors of ind, then only label one of them.
tmp2=ismember(expairs,neighbors);
tmp3=find(sum(transpose(tmp2))==2);
if (length(tmp3)>0)
exneighbors=union(exneighbors,transpose(expairs(tmp3,2)));
end

% Recursively visit neighbors.
% Exclude exneighbors.
tmp=setdiff(neighbors,exneighbors);


% If a new neighbor forms expairs with the nodes of the current component, then discard the neighbor.
sel=ones(1,length(tmp));
for i=1:length(tmp)
j=tmp(i); tmp3=[];
tmp2=find(expairs(:,1)==j);
if (length(tmp2)>0)
tmp3=union(tmp3,expairs(tmp2,2));
end
tmp2=find(expairs(:,2)==j);
if (length(tmp2)>0)
tmp3=union(tmp3,expairs(tmp2,1));
end
if (length(tmp3)>0)
if (sum(labeled(tmp3)==label)>0)
sel(i)=0;
end
end
end

tmp=tmp(find(sel==1));


if (length(tmp)>0)

% If all neighbors are labeled, then return.
if (sum(labeled(tmp)==0)>0)
for i=1:length(tmp)
if (labeled(tmp(i))==0)
labeled=recurse_label3(tmp(i),nnodes,G,labeled,label,nexpairs,expairs);
end
end
end

end
